# main.py
from flask import Flask
from .api_case import case_routes
from .api_test import test_routes
from flask_cors import CORS

app = Flask(__name__)

# CORS(app)
CORS(app, resources=r'/*', supports_credentials=True)

# register routes
app.register_blueprint(case_routes, url_prefix='/api/case')
app.register_blueprint(test_routes, url_prefix='/api/test')


# src/main.py
@app.after_request
def add_header(response):
    return response



"""
entry point for third party to call
afford 2 calling choices
"""

# """
# choice 1
# same as executing command `pyland` at term, example following:
# ❯ python main.py
# usage: pyland [-h] [--version] [--no-color] {init,server,platform}
# pyland: error: the following arguments are required: command, args
# """
#
# if __name__ == "__main__":
#     import pyland
#     sys.exit(pyland.main())


"""
choice 2
same as executing command `pyland platform` at term
❯ python main.py
usage: Process some integrated operations with Auto platform.
       [-h] (--import IMPORT | --list LIST | --run RUN)
Process some integrated operations with Auto platform.: error: one of the arguments --import --list --run is required
"""

if __name__ == "__main__":
    import pyland
    import sys

    sys.exit(pyland.main(['platform', *sys.argv[1:]]))



# coding=utf8
from .config import Config
from .utils.client import HTTPClient
from .log import logger
from .extractor import extract


class PRequest(object):
    def __init__(self, base_url='', conf='config.yml', host='host_url'):
        if base_url:
            self.base_url = base_url
        else:
            try:
                base_url = Config(conf).get(host, index=0)
                if base_url:
                    self.base_url = base_url
            except Exception as e:
                pass
        self.headers: dict = {}

    def send_request(self, api_url, method='POST', status='PASS', params=None, data=None, json=None, files=None,
                     extractor=None, timeout=10, cookies=None):
        """请求定制封装"""
        if not hasattr(self, 'logger'):
            self.logger = logger
        if not hasattr(self, 'headers'):
            self.headers: dict = {}

        # 建立http连接
        if not self.headers:
            client = HTTPClient(url=(self.base_url + api_url), method=method, timeout=timeout, cookies=cookies)
        else:
            client = HTTPClient(url=(self.base_url + api_url), method=method, headers=self.headers, timeout=timeout,
                                cookies=cookies)
        # 发送请求
        client.logger = self.logger
        response = client.send(params=params, json=json, data=data, files=files)

        try:
            # 有效用例和无效用例的最基本断言（针对status_code）
            if status == 'PASS':
                assert response.status_code in [200]
            elif status == 'FAIL':
                assert response.status_code not in [200]

            # 提取json值
            if status == 'PASS' and extractor is not None:
                response = extract(extractor, response.text)
                self.logger.info("数据处理: 提取{}的值为{} ".format(extractor, response))
        except Exception as e:
            self.logger.error(e)
        client.close()
        return response



import argparse
from typing import List
from pyland import main_scaffold


def main(args: List[str]) -> bool:
    parser = argparse.ArgumentParser(prog="pyland init")
    parser.add_argument(
        "--simple",
        action="store_true",
        default=False,
        required=False,
        help="generate simple demo project by project name",
    )
    parser.add_argument(
        "--sub",
        action="store_true",
        default=False,
        required=False,
        help="sub project has no conftest.py and main.py",
    )
    parser.add_argument(
        "--topo",
        action="store_true",
        default=False,
        required=False,
        help="topo project format services.py and test_01.py",
    )
    parser.add_argument(
        "--project",
        default='demo',
        required=False,
        help="generate demo project by project name",
    )

    parsed_args = parser.parse_args(args)

    return main_scaffold(project=parsed_args.project,
                         simple=parsed_args.simple,
                         sub=parsed_args.sub,
                         topo=parsed_args.topo
                         )



# coding=utf8
from re import findall, sub
from ..sql import Sql


class TableCount(object):
    def __init__(self, sql_server):
        self.sql_server = sql_server

    def get_table_list(self, table_name):
        # 根据数据库配置，获取指定名称开头的表集合，组成列表
        query_table_list = "show tables;"
        sqler = Sql(self.sql_server)
        res = sqler.query(query_table_list)
        sqler.close()

        _list = findall('(\'{}.*?\')'.format(table_name),str(res))
        if _list:
            table_list = [sub("'",'',each) for each in _list]
            # logger.info(table_list)
        else:
            table_list = []
        return table_list


    def query_table_count(self, table_list, condition=""):
        # 根据数据库配置，以及表集合，循环查询每个表的数据条数
        table_info = []
        table_count = 0
        if table_list:
            for i in table_list:
                query_count = "SELECT COUNT(ID) FROM {} {};".format(i, condition)

                sqler = Sql(self.sql_server)
                res = int(sqler.query(query_count)[0][0])
                sqler.close()

                table_info.append((i, res))
                table_count += res
        return table_info, table_count


# coding=utf8
import pytest
import os
import sys
from .utils.allure_opt import copy_history
from .log import logger


# *按需更改*
RUN_PARAM = {
    "collect": ['test_suits', '--collect-only'],
    "by_dir": ['test_suits'],
    "by_mod": ['test_suits/test_01_demo.py'],
    "by_class": ['test_suits/test_01_demo.py::TestDemo'],
    "by_func": ['test_suits/test_01_demo.py::TestDemo::test_demo_valid'],
    "by_search": ['-k', 'test_demo_valid'],
    "by_severity": ['--allure-severities', 'blocker'],
    "by_feature": ['--allure-features', 'Demo'],
    "by_story": ['--allure-stories', 'List Demo'],
}

# 不需更改
ALLURE_PARAM = [
    '-v',
    '--alluredir',
    'allure-result',
    "--clean-alluredir"
]


def run_by(run_param=None):
    # 切换工作空间
    BASE_PATH = os.path.abspath(sys.path[0])
    os.chdir(f'{BASE_PATH}')
    logger.info(f"current workspace: {BASE_PATH}")

    # *按需调用*
    if run_param and type(run_param) is list:
        pytest_param = run_param + ALLURE_PARAM
    else:
        logger.warning(f"not read valid run param, collect from current workspace {BASE_PATH}")
        pytest_param = ALLURE_PARAM

    # 执行测试用例
    pytest.main(pytest_param)

    # 复制历史结果，并生成报告
    # copy_history("report", "allure-result")
    os.system(f"allure generate allure-result -o report --clean")

    # 打开报告， 生成url
    os.system(f"allure open report")


"""一些生成器方法，生成随机数，手机号，以及连续数字等"""
import random
import os
from faker import Factory
from time import time as ttm
from PIL import Image
import png
from concurrent.futures import ThreadPoolExecutor
from ..config import Config


fake = Factory().create('zh_CN')


def random_phone_number():
    """随机手机号"""
    return fake.phone_number()


def random_name():
    """随机姓名"""
    return fake.name()


def random_address():
    """随机地址"""
    return fake.address()


def random_email():
    """随机email"""
    return fake.email()


def random_ipv4():
    """随机IPV4地址"""
    return fake.ipv4()


def random_str(min_chars=0, max_chars=8):
    """长度在最大值与最小值之间的随机字符串"""
    return fake.pystr(min_chars=min_chars, max_chars=max_chars)


def factory_generate_ids(starting_id=1, increment=1):
    """ 返回一个生成器函数，调用这个函数产生生成器，从starting_id开始，步长为increment。 """

    def generate_started_ids():
        val = starting_id
        local_increment = increment
        while True:
            yield val
            val += local_increment

    return generate_started_ids


def factory_choice_generator(values):
    """ 返回一个生成器函数，调用这个函数产生生成器，从给定的list中随机取一项。 """

    def choice_generator():
        my_list = list(values)
        # rand = random.Random()
        while True:
            yield random.choice(my_list)

    return choice_generator


def gen_16_randint(total=1, uniq_flag=1):
    """
    随机生成16位数字，跟当前时间戳有关(1毫秒100个数，如果重复，叠加头部标志）
    :param total: 随机数个数
    :param uniq_flag: 头部标志，内部调用
    :return: 返回列表
    """
    uniq_flag = 1 if not uniq_flag else int(uniq_flag)
    total = 1 if not total else int(total)
    # 每0.1ms生成10个数
    now_time = lambda: int(round(ttm() * 10000))
    num_list = ["{}{}{}".format(uniq_flag, now_time(), i) for i in range(10)]
    # 递归返回随机值
    if 0 < total < 10:
        num_list = random.sample(num_list, total)
    elif total > 10:
        num_list.extend(gen_16_randint(total - 10))
        # 递归去重
        unique_count = len(set(num_list))
        if unique_count != len(num_list):
            num_list = list(set(num_list))
            uniq_flag = uniq_flag + 1 if uniq_flag < 9 else 1
            num_list.extend(gen_16_randint(total - unique_count, uniq_flag))
    return num_list


def gen_17_randint(total=1, uniq_flag=1):
    """
    随机生成17位数字，跟当前时间戳有关(1毫秒1000个数，如果重复，叠加头部标志）
    :param total: 随机数个数
    :param uniq_flag: 头部标志，内部调用
    :return: 返回列表
    """
    uniq_flag = 1 if not uniq_flag else int(uniq_flag)
    total = 1 if not total else int(total)
    # 每0.1ms生成10个数
    now_time = lambda: int(round(ttm() * 10000))
    num_list = [int("{}{}{:02}".format(uniq_flag, now_time(), i)) for i in range(100)]
    # 递归返回随机值
    if 0 < total < 100:
        num_list = random.sample(num_list, total)
    elif total > 100:
        num_list.extend(gen_17_randint(total - 100))
        # 递归去重
        unique_count = len(set(num_list))
        if unique_count != len(num_list):
            num_list = list(set(num_list))
            uniq_flag = uniq_flag + 1 if uniq_flag < 9 else 1
            num_list.extend(gen_17_randint(total - unique_count, uniq_flag))
    return num_list


def gen_image(filename='default', size=(1280, 720), color=(255, 255, 255), format='JPEG'):
    """
    size: (1280, 720)
    color: (255, 255, 255)
    format: "PNG"
    """
    img = Image.new('RGB', size, color)
    img.save(filename, format)


def gen_png(width, hight, size, path=None):
    if width > 800 or hight > 800:
        raise ValueError("width or height max is 800")
    if not path:
        path = Config().DATA_PATH
    s = []
    for i in range(hight):
        s.append([])
        for j in range(width):
            s[i].append(0)

    palette = [(0x55, 0x55, 0x55), (0xff, 0x99, 0x99)]
    w = png.Writer(width, hight, palette=palette, bitdepth=1)
    filename = os.path.join(path, f'{width}x{hight}_{size}k.png')
    f = open(filename, 'wb')

    with ThreadPoolExecutor(max_workers=15) as executor:
        args = ((f, s) for i in range(size * 10))
        for result in executor.map(lambda p: w.write(*p), args):
            pass


if __name__ == '__main__':
    print(random_phone_number())
    print(random_name())
    print(random_address())
    print(random_email())
    print(random_ipv4())
    print(random_str(min_chars=6, max_chars=8))
    id_gen = factory_generate_ids(starting_id=0, increment=2)()
    for i in range(5):
        print(next(id_gen))

    choices = ['John', 'Sam', 'Lily', 'Rose']
    choice_gen = factory_choice_generator(choices)()
    for i in range(5):
        print(next(choice_gen))

    # gen_image("10x10.png", (10, 10), (255, 255, 255), "PNG")
    # gen_png(800, 400, 100)
    # gen_png(800, 800, 100)
    # gen_png(79, 79, 1151)
    # gen_png(800, 400, 100)
    gen_image('1280x960_1M', (1280, 960))


# coding=utf8
"""
execute cases and extract results
input:
{
    # "env": {},
    "project_name": "DEMO",
    "project_path": "DEMO",
    "output_path": "",
    "cases": [{
        "case_id": 1,
        # "case_name": "",
        # "case_category": "",
        # "project_name": "",
        # "output_path": "",
        "extra": {}
    }]
}
output:
{
    "report_file": "",
    "log_file": "",
    "cases": [
        {
            "case_id": None,
            "case_status": "string",
            "variables": {
                "in_puts": {},  # 数据输入
                "out_puts": {}  # 数据输出（供后面用例使用）
            },
            "attachment": {
                "log_file": "string",  # 详细执行过程文件路径
                "html_file": "string"  # html报告文件路径
            },
            "steps": [{
                "name": "string",  # 测试步骤名字
                "status": "string",  # 测试步骤状态（pass/fail/error）
                "request": {},  # 请求
                "response": {},  # 响应
                "validators": [{  # 校验项
                    "comparator": "equal",
                    "check": "status_code",
                    "check_value": None,
                    "expect_value": None,
                    "message": None,
                    "check_result": None
                }],
                "erroes": "error msg"
            }]
        }
    ]
}
"""
CASE_LIST_JSON = {
    "env": {},
    "project_name": "DEMO",
    "project_path": "DEMO",
    "output_path": "/Users/jiukunzhang/PycharmProjects/dl_cms/tmp/output.json",
    "cases": [
        {
            "case_id": 1,
            "case_name": "ListDemoInValid",
            "case_category": "PY",
            "project_name": "DEMO",
            "output_path": "/Users/jiukunzhang/PycharmProjects/dl_cms/tmp/1",
            "extra": {
                'class': "TestDemo",
                'function': 'test_demo_invalid',
                'module': 'tests.DEMO.test_suits.test_01_demo',
                'nodeID': 'DEMO/test_suits/test_01_demo.py: : TestDemo: : test_demo_invalid',
                'path': 'DEMO/test_suits/test_01_demo.py',
                'severity': 'blocker',
                'story': 'ListDemo'
            }
        },
        {
            "case_id": 2,
            "case_name": "ListDemoValid",
            "case_category": "PY",
            "project_name": "DEMO",
            "output_path": "/Users/jiukunzhang/PycharmProjects/dl_cms/tmp/1",
            "extra": {
                'class': "TestDemo",
                'function': 'test_demo_valid',
                'module': 'tests.DEMO.test_suits.test_01_demo',
                'nodeID': 'DEMO/test_suits/test_01_demo.py: : TestDemo: : test_demo_valid1',
                'path': 'DEMO/test_suits/test_01_demo.py',
                'severity': 'blocker',
                'story': 'ListDemo'
            }
        }]
}

CASE_RESULTS_JSON = {
    "report_file": "",
    "log_file": "",
    "cases": [
        {
            "case_id": None,
            "case_status": "passed",
            "variables": {
                "in_puts": {},  # 数据输入
                "out_puts": {}  # 数据输出（供后面用例使用）
            },
            "attachment": {
                "log_file": "",  # 详细执行过程文件路径
                "html_file": ""  # html报告文件路径
            },
            "steps": [{
                "name": "step 1",  # 测试步骤名字
                "status": "passed",  # 测试步骤状态（pass/fail/error）
                "request": {},  # 请求
                "response": {},  # 响应
                "validators": [{  # 校验项
                    "comparator": "equal",
                    "check": "status_code",
                    "check_value": None,
                    "expect_value": None,
                    "message": None,
                    "check_result": None
                }],
                "erroes": "error msg"
            }]
        }
    ]
}

import os
import sys
import re
from subprocess import Popen, PIPE
from copy import deepcopy
import shutil
from .config import Config, BASE_PATH
from .log import Logger
from .extractor import extract
from .cases_collect import UpdateCase
from .utils.allure_opt import copy_history
from .utils.support import rec_merge
from .utils.error import MissingError

__all__ = [
    "ExecTests",
    "run_case"
]


class ExecTests(object):
    """批量执行测试用例，生成指定格式json，保存在指定位置"""

    def __init__(self, case_list_json):
        """
        0 获取基础路径，准备执行环境
        """
        try:
            # 项目基本信息
            project_path = extract("project_path", case_list_json)
            self.output_path = extract("output_path", case_list_json)
            self.project_path = os.path.abspath(os.path.join(BASE_PATH, project_path))
            self.runtime_args = extract("runtime_args", case_list_json)
            if os.path.exists(self.project_path):
                sys.path.extend([BASE_PATH, self.project_path])
                # os.chdir(self.project_path)
            else:
                raise FileNotFoundError(f"path not exists: {self.project_path} ")
        except KeyError:
            raise MissingError("missing required parameters for 'project_path'")
        except FileNotFoundError as e:
            raise FileNotFoundError(f"path not exists: {self.project_path} ")

        # 数据路径
        cfg = Config(path=self.project_path)
        self.data_path = cfg.DATA_PATH
        self.log_path = cfg.LOG_PATH
        self.report_path = cfg.REPORT_PATH
        self.result_path = cfg.RESULT_PATH

        self.log_config = cfg.config.get('log') if cfg.config else None
        self.logger = Logger(log_path=self.log_path, config=self.log_config)

    def get_case_nodes(self, case_list_json):
        """
        1 提取nodeID信息，供批量执行
        return:
        {
            "DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_valid":
                {"case_id": 123}
        }
        """
        case_nodes = {}
        case_list = extract("cases", case_list_json)

        # real_node_list = UpdateCase(case_list_json).get_case_node_list()
        # self.logger.debug(f"real_node_list: {real_node_list} \n\n")
        # for case in case_list:
        #     case_id = extract("case_id", case)
        #     nodeID = extract("extra.nodeID", case).strip().replace(": ", ":")
        #     if nodeID in real_node_list:
        #         case_nodes.update({nodeID: {"case_id": case_id}})
        #     else:
        #         case_nodes.update({nodeID: {"case_id": case_id, "case_status": "NotFound"}})
        #
        # self.logger.info(f"case_nodes_id: {case_nodes} \n\n")
        # return case_nodes

        for case in case_list:
            case_id = extract("case_id", case)
            nodeID = extract("extra.nodeID", case).strip().replace(": ", ":")
            case_nodes.update({nodeID: {"case_id": case_id}})

        self.logger.info(f"case_nodes_id: {case_nodes} \n\n")
        return case_nodes

    def run_test(self, case_nodes):
        """2 运行测试用例"""
        try:
            # 1. backup pytets.ini
            if os.path.exists("pytest.ini"):
                shutil.copy("pytest.ini", "pytest.ini.bak")
            # 2. update pytest.ini by case_nodes
            self._pytest_config(case_nodes)
            # 3. execute testcases by pytets.ini
            self._run_pytest_cases()
        except Exception as e:
            self.logger.error(e)
        finally:
            # 4. recovery pytest.ini
            if os.path.exists("pytest.ini.bak") and os.path.exists("pytest.ini"):
                shutil.move("pytest.ini.bak", "pytest.ini")

        # 5. get execution results
        case_results = self._get_case_results(case_nodes)
        # rec_merge(case_results, case_nodes)

        # 6. generate reports history
        # self._history_allure_report()

        return case_results

    def _pytest_config(self, case_nodes):
        """2.1 把nodeid写到pytest.ini，批量执行(*跳过NotFound的用例*）"""
        # os.chdir(self.project_path)
        with open("pytest.ini", "w") as f:
            f.write('[pytest]\ntestpaths =\n')
            for node, node_info in case_nodes.items():
                # node = re.sub(r'.*/test_suits/', 'test_suits/', node)
                # if node_info.get("case_status") != 'NotFound':
                f.write("\n\t" + node)

    def _run_pytest_cases(self):
        """2.2 批量执行测试用例, 生成allure报告"""
        PYTEST_PARAM = ['python', '-m', 'pytest']
        ALLURE_PARAM = ['--alluredir', self.result_path, "--clean-alluredir"]

        if self.runtime_args and isinstance(self.runtime_args, dict):
            for k, v in self.runtime_args.items():
                PYTEST_PARAM.extend([k, v])
        else:
            PYTEST_PARAM.extend(['--tb=short', '-n', 'auto'])

        RUNCMD = PYTEST_PARAM + ALLURE_PARAM
        self.logger.info(RUNCMD)

        result_bak_path = self.result_path + '.bak'
        if os.path.exists(result_bak_path):
            shutil.rmtree(result_bak_path)

        if os.path.exists(self.result_path):
            shutil.move(self.result_path, result_bak_path)

        if not os.path.exists(self.report_path):
            os.makedirs(self.report_path)

        with Popen(RUNCMD,
                   stdout=PIPE,
                   bufsize=1,
                   universal_newlines=True,
                   stderr=PIPE
                   ) as p:
            for line in p.stderr:
                print(line)

        os.system(f"allure generate {self.result_path} -o {self.report_path} --clean")

        if os.path.exists(self.result_path):
            shutil.rmtree(self.result_path)

        if os.path.exists(result_bak_path):
            shutil.move(result_bak_path, self.result_path)

    def _get_case_results(self, case_nodes):
        """
        2.3 获取用例执行结果
        return:
        {
            "DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_valid":
                {
                    "case_status": 'passed',
                    "steps": [
                        {"name": "", "case_status": "passed"},
                        ]
                }
        }
        """
        REPORT_CASE_PATH = os.path.join(self.report_path, "data/test-cases")

        if not os.path.exists(REPORT_CASE_PATH):
            return f"Error: test case excuted block, no report file generated"

        case_result = {}
        case_result_simple = {}

        for c in os.listdir(REPORT_CASE_PATH):
            res = Config(os.path.join(REPORT_CASE_PATH, c)).json_get()
            name = extract("name", res)
            case_path = res["fullName"]
            # self.logger.debug(f"{case_path}\n\n")
            nodeID = self._convert_allure_to_pytest_nodeid(case_path)

            if not nodeID:
                continue
            if case_nodes.get(nodeID):
                _node = case_nodes.get(nodeID)
                case_id = extract("case_id", _node)
            case_status = extract("testStage.status", res)

            step_result = deepcopy(extract("cases[0].steps[0]", CASE_RESULTS_JSON))
            step_result.update({"name": name, "status": case_status})

            if not case_result.get(nodeID):
                node_result = deepcopy(extract("cases[0]", CASE_RESULTS_JSON))
                node_result.update({"case_status": case_status, "case_id": case_id, "steps": [step_result]})
                case_result[nodeID] = node_result
            else:
                case_result[nodeID]['steps'].append(step_result)
                if case_status != 'passed': case_result[nodeID]['case_status'] = 'failed'

        for node in case_result:
            self._translate_status(case_result[node])
        # self.logger.debug(f"case_nodes_status: {case_result} \n\n")

        for key, value in case_result.items():
            node = {}
            node['steps'] = []
            node['case_id'] = value.get('case_id')
            node['case_status'] = value.get('case_status')
            for i in value.get('steps'):
                if type(i) is dict:
                    node['steps'].append(i.get('status'))
            case_result_simple[key] = node

        self.logger.info(f"case status: {case_result_simple} \n\n")
        return case_result

    def _translate_status(self, case):
        """
        change status passed to pass, failed to fail
        case:
        {
            "DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_valid":
                {
                    "case_status": 'passed',
                    "steps": [
                        {"name": "", "status": "passed"},
                        ]
                }
        }
        """
        if case['case_status'] == 'passed':
            case['case_status'] = 'pass'
        elif case['case_status'] == 'failed':
            case['case_status'] = 'fail'
        else:
            pass
        if 'steps' in case.keys():
            for step in case['steps']:
                if step['status'] == 'passed':
                    step['status'] = 'pass'
                elif step['status'] == 'failed':
                    step['status'] = 'fail'
                else:
                    pass

    def _convert_allure_to_pytest_nodeid(self, case_path):
        """ 2.3.1
        case_path: 'DEMO.test_suits.test_01_demo.TestDemo#test_demo_valid'
        to
        case_nodeid: 'DEMO/test_suits/test_01_demo.py::TestDemo::test_demo_valid'
        """
        _info = case_path.split("#")
        if len(_info) != 2:
            self.logger.error("can not parse case node")
            return None

        _module, func = _info
        _path = _module.split(".")

        if _path[-1].startswith("Test"):
            cls = _path[-1]
            module = _path[-2]
            path = _path[:-2]
            case_nodeid = '/'.join(path) + '/' + module + '.py' + "::" + cls + "::" + func
        else:
            case_nodeid = _module.replace('.', '/') + '.py' + "::" + func
        # self.logger.debug(f"case_nodeid: {case_nodeid}")
        return case_nodeid

    # def _history_allure_report(self):
    #     copy_history(self.report_path, self.result_path)
    #     os.system(f"allure generate {self.result_path} -o {self.report_path} --clean")

    def export_json(self, case_result, log_file_name):
        """
        3. 返回指定格式json输出到指定路径
        """
        output_json = deepcopy(CASE_RESULTS_JSON)

        output_json["report_file"] = self.report_path
        output_json["log_file"] = log_file_name
        output_json["cases"] = list(case_result.values())

        # self.logger.debug(f"output_json: {output_json}")
        Config(self.output_path).json_put(output_json)
        return output_json

    def backup_log(self):
        self.logger.info("备份日志")
        log_file_name = self.logger.log_file_name
        log_file_name_bak = f"{self.logger.log_file_name}.bak"
        os.system(
            f"touch {log_file_name} {log_file_name_bak}; "
            f"cat {log_file_name} >> {log_file_name_bak}; "
            f"cat /dev/null > {log_file_name}"
        )
        return log_file_name


def run_case(case_list_json=CASE_LIST_JSON):
    et = ExecTests(case_list_json)
    log_file_name = et.backup_log()
    case_nodes = et.get_case_nodes(case_list_json)
    case_results = et.run_test(case_nodes)
    res = et.export_json(case_results, log_file_name)
    return res


if __name__ == "__main__":
    """只需要读取一个输入参数，定义名称为 CASE_LIST_JSON，就可以了"""
    run_case(CASE_LIST_JSON)



"""
在这里添加各种自定义的断言，断言失败抛出AssertionError就OK。
"""
from collections import Counter
from ..log import logger
from ..extractor import extract


def assertHTTPCode(response, code_list=None):
    res_code = response.status_code
    if not code_list:
        code_list = [200]
    if res_code not in code_list:
        raise AssertionError('响应code不在列表中！')  # 抛出AssertionError，unittest会自动判别为用例Failure，不是Error


def assert_search_api_pagination(func, param, ext_key="result.images[].id", ext_total="result.total"):
    """
    assert search apis with param offset and limit
    1. assert different offset should not has same response
    2. assert all pages num = total num
    api request example:
    {
        'offset': 0,
        'limit': 10
    }
    api response example:
    {
    'result':
        {
        'images': [
            {'id': 1},
            {'id': 2}
        ],
        'total': 2
        }
    }
    """
    response = func(param)
    total = extract(ext_total, response)
    pagination = 10 if total > 10 else 1

    num = 0
    res_all = []
    err_all = []
    for i in range(0, total, pagination):
        param.update({'limit': pagination, 'offset': i})
        res_id = extract(ext_key, func(param))
        logger.info(f'limit: {pagination}, offset: {i}, {ext_key} : {res_id}')
        res_all.extend(res_id)
        num += len(res_id)

    for r in Counter(res_all).items():
        logger.info(r)
        if r[1] > 1:
            err_all.append(r[0])
        else:
            continue
    logger.info(f"all results：{res_all}")
    if err_all:
        logger.error(f"duplicate results：{err_all}")
    assert total == num
    assert err_all == []



"""Command line options."""
import os
import sys
import py
import argparse
from typing import Any, Optional, Sequence, Union, List
from importlib_metadata import entry_points
import requests
import colorama
import http
import json
import pyland
from importlib_metadata import version

args = argparse.Namespace()


def main(argv: Optional[Union[py.path.local, List[str]]] = None) -> Any:
    if argv is None:
        argv = sys.argv[1:]
    elif isinstance(argv, py.path.local):
        argv = [str(argv)]
    elif not isinstance(argv, list):
        msg = "`argv` parameter expected to be a list of strings, got: {!r} (type: {})"
        raise TypeError(msg.format(argv, type(argv)))

    try:
        result = dispatch(argv)
    except requests.HTTPError as exc:
        status_code = exc.response.status_code
        status_phrase = http.HTTPStatus(status_code).phrase
        result = (
            f"{exc.__class__.__name__}: {status_code} {status_phrase} "
            f"from {exc.response.url}\n"
            f"{exc.response.reason}"
        )
    except pyland.exceptions.PylandException as exc:
        result = f"{exc.__class__.__name__}: {exc.args[0]}"

    return _format_error(result) if isinstance(result, str) else result


def console_main() -> int:
    """The CLI entry point of pyland.

    This function is not meant for programmable use; use `main()` instead.
    """
    # https://docs.python.org/3/library/signal.html#note-on-sigpipe
    try:
        code = main()
        sys.stdout.flush()
        return code
    except BrokenPipeError:
        # Python flushes standard streams on exit; redirect remaining output
        # to devnull to avoid another BrokenPipeError at shutdown
        devnull = os.open(os.devnull, os.O_WRONLY)
        os.dup2(devnull, sys.stdout.fileno())
        return 1  # Python exits with error code 1 on EPIPE


def dispatch(argv: List[str]) -> Any:
    registered_commands = entry_points(group="pyland.registered_commands")
    parser = argparse.ArgumentParser(prog="pyland")
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s version {} ".format(pyland.__version__),
    )
    parser.add_argument(
        "--no-color",
        default=False,
        required=False,
        action="store_true",
        help="disable colored output",
    )
    parser.add_argument(
        "command",
        choices=registered_commands.names,
    )
    parser.add_argument(
        "args",
        help=argparse.SUPPRESS,
        nargs=argparse.REMAINDER,
    )

    parser.parse_args(argv, namespace=args)

    main = registered_commands[args.command].load()

    return main(args.args)


class EnvironmentDefault(argparse.Action):
    """Get values from environment variable."""

    def __init__(
            self,
            env: str,
            required: bool = True,
            default: Optional[str] = None,
            **kwargs: Any,
    ) -> None:
        default = os.environ.get(env, default)
        self.env = env
        if default:
            required = False
        super().__init__(default=default, required=required, **kwargs)

    def __call__(
            self,
            parser: argparse.ArgumentParser,
            namespace: argparse.Namespace,
            values: Union[str, Sequence[Any], None],
            option_string: Optional[str] = None,
    ) -> None:
        setattr(namespace, self.dest, values)


class EnvironmentFlag(argparse.Action):
    """Set boolean flag from environment variable."""

    def __init__(self, env: str, **kwargs: Any) -> None:
        default = self.bool_from_env(os.environ.get(env))
        self.env = env
        super().__init__(default=default, nargs=0, **kwargs)

    def __call__(
            self,
            parser: argparse.ArgumentParser,
            namespace: argparse.Namespace,
            values: Union[str, Sequence[Any], None],
            option_string: Optional[str] = None,
    ) -> None:
        setattr(namespace, self.dest, True)

    @staticmethod
    def bool_from_env(val: Optional[str]) -> bool:
        """Allow '0' and 'false' and 'no' to be False."""
        falsey = {"0", "false", "no"}
        return bool(val and val.lower() not in falsey)


def _format_error(message: str) -> str:
    pre_style, post_style = "", ""
    if not args.no_color:
        colorama.init()
        pre_style, post_style = colorama.Fore.RED, colorama.Style.RESET_ALL

    return f"{pre_style}{message}{post_style}"



"""
[Important!!!  DO NOT TOUCH!!!]
"""
import os
from pyland import Config


pytest_plugins = 'pytester'


def pytest_addoption(parser):
    parser.addoption('--case-manage', required=False, help='analysis cases to manage by platform')


def pytest_generate_tests(metafunc):
    res = {}
    res['extra'] = {}
    res['extra']['params'] = {}

    for param in metafunc.fixturenames:
        if param.startswith('param'):
            if not param.endswith('valid'):
                param = param + '_valid'
            if hasattr(metafunc.module, 'combined'):
                param_pool = metafunc.module.combined[param]
                metafunc.parametrize(param, param_pool)
                # res['extra']['params'][param] = param_pool
            else:
                raise ValueError(f"param {param} set, but not found `combined`")
            
    project_path = metafunc.config.getoption('--case-manage')

    if project_path:
        for mark in metafunc.definition.iter_markers():
            if mark.name == 'allure_display_name':
                res['name'] = mark.args[0]
            elif mark.name == 'allure_label':
                label_type = mark.kwargs['label_type']
                label_value = mark.args[0]
                res['extra'][label_type] = label_value

        res['extra']['nodeID'] = metafunc.definition.nodeid
        res['extra']['path'] = metafunc.definition.location[0]
        res['extra']['module'] = metafunc.module.__name__
        if hasattr(metafunc.module, '__author__'):
            res['author'] = metafunc.module.__author__
        else:
            res['author'] = 'has no author yet'
        res['extra']['class'] = metafunc.cls.__name__ if metafunc.cls else None
        res['extra']['function'] = metafunc.function.__name__
        res['description'] = 'has no description yet' if not metafunc.function.__doc__ else metafunc.function.__doc__
        res['primary_key'] = 'extra.nodeID'
        res['category'] = 'pyland'
        res['priority'] = res['extra'].get('severity') if res['extra'].get('severity') else ''

        cfg = Config(path=project_path)
        case_manage_yml = os.path.join(cfg.DATA_PATH, '.tmp_case_manage.yml')
        cfg.update([res], case_manage_yml)




# coding=utf8
"""
generate cases mapping relationship
upstream param:
{
    "project_id": "",
    "project_path": ""
}
call case/many api param:
[
  {
    "primary_key": "anyitem or extra.xxxx",
    "author": "string",
    "name": "string",
    "category": "string",
    "priority": "string",
    "description": "string",
    "project_id": 0,
    "group_id": 0,
    "extra": {}
  }
]
"""
PROJECT_JSON = {
    "project_id": 35,
    "project_path": "DEMO"
}

import os
import sys
import shutil
from subprocess import Popen, PIPE
from .config import Config, BASE_PATH
from .log import Logger
from .extractor import extract
from _pyland.thirdparty import CaseManage
from .utils.error import MissingError

__all__ = [
    "UpdateCase",
    "list_case",
    "update_case"
]


PYTESTCONFIG = 'pytest.ini'
PYTESTCONFIG1 = 'pytest.ini.bak'


class UpdateCase(object):
    """批量更新用例到平台"""

    def __init__(self, project_json):
        """
        0 获取基础路径，准备执行环境
        """
        try:
            # 项目基本信息
            self.project_id = project_json.get("project_id")
            self.platform_url = project_json.get("host")
            project_path = project_json["project_path"]
            self.project_path = os.path.abspath(os.path.join(BASE_PATH, project_path))
            if os.path.exists(self.project_path):
                sys.path.extend([BASE_PATH, self.project_path])
                # os.chdir(self.project_path)")
        except FileNotFoundError as e:
            raise FileNotFoundError(f"path not exists: {self.project_path} ")

        # 数据路径
        cfg = Config(path=self.project_path)
        self.data_path = cfg.DATA_PATH
        self.log_path = cfg.LOG_PATH
        self.logger = Logger(log_path=self.log_path)

    def get_case_base_info(self, project_path=None):
        """
        1. 获取用例映射关系等基本信息， 以nodeID为判断基准
        """
        project_path = project_path if project_path else self.project_path
        case_base_info = []
        self.logger.info(f"{os.getcwd()}")

        if os.path.exists(PYTESTCONFIG):
            shutil.copy(PYTESTCONFIG, PYTESTCONFIG1)

        with open(PYTESTCONFIG, "w") as f:
            f.write('[pytest]\ntestpaths =\n\t' + project_path)

        with Popen(['python', '-m', 'pytest',
                    '--collect-only',
                    '--tb=short',  # shorter traceback format
                    '--case-manage='+project_path,
                    ], stdout=PIPE, bufsize=1,
                   universal_newlines=True) as p:
            for line in p.stdout:
                print(line, end='')

        if os.path.exists(PYTESTCONFIG1) and os.path.exists(PYTESTCONFIG):
            shutil.move(PYTESTCONFIG1, PYTESTCONFIG)

        case_manage_yml = os.path.join(self.data_path, ".tmp_case_manage.yml")
        if os.path.exists(case_manage_yml):
            case_base_info = Config(case_manage_yml).get()
            os.remove(case_manage_yml)
        self.logger.debug(f"case_base_info: {case_base_info}")
        return case_base_info

    def get_case_info(self, case_base_info=None):
        """
        2. 补全case信息（根据case的三级模块，判断并创建三级分组;暂时不用）
        """
        # for case in case_base_info:
        #     epic = case["extra"].get("epic")
        #     feature = case["extra"].get("feature")
        #     story = case["extra"].get("story")
        #
        #     mum_id = 1
        #     for group in [epic, feature, story]:
        #         project_name =
        if not case_base_info:
            case_base_info = self.get_case_base_info()
        for case in case_base_info:
            case.update({"project_id": self.project_id})
        case_info = case_base_info
        self.logger.info(f"本次发现{len(case_info)}条测试用例")
        return case_info

    def update_case_to_platform(self, case_info, platform_url=''):
        """
        3 更新用例到平台
        """
        if not platform_url:
            platform_url = self.platform_url
        cm = CaseManage(platform_url)
        res = cm.import_case(cases=case_info)
        return res

    def get_case_node_list(self, project_path=None):
        """
        4 获取用例的nodeid列表, 仅供用例执行的查询
        """
        project_path = project_path if project_path else self.project_path

        case_base_info = self.get_case_base_info(project_path)
        case_node_list = []
        for case in case_base_info:
            nodeID = extract("extra.nodeID", case).strip().replace(": ", ":")
            case_node_list.append(nodeID)
        return case_node_list


def update_case(project_json=PROJECT_JSON):
    uc = UpdateCase(project_json)
    uc.logger.info(project_json)
    case_info = uc.get_case_info()
    res = uc.update_case_to_platform(case_info)
    return res


def list_case(project_json=PROJECT_JSON):
    uc = UpdateCase(project_json)
    uc.logger.info(project_json)
    case_info = uc.get_case_info()
    return case_info


if __name__ == "__main__":
    """传入PR_LIST_JSON = {project_path、project_id、group_id}就可以了"""
    # update_case(PROJECT_JSON)
    list_case(PROJECT_JSON)



# coding=utf8
from pyland import run_by

# *按需更改*
RUN_PARAM = {
    "collect": ['test_suits', '--collect-only'],
    "by_dir": ['test_suits'],
    "by_mod": ['test_suits/test_01.py'],
    "by_class": ['test_suits/test_01.py::TestDemo'],
    "by_func": ['test_suits/test_01.py::TestDemo::test_demo_valid'],
    "by_search": ['-k', 'test_demo_valid'],
    "by_severity": ['--allure-severities', 'blocker'],
    "by_feature": ['--allure-features', 'Demo'],
    "by_story": ['--allure-stories', 'List Demo'],
}

if __name__ == "__main__":
    # *按需调用*
    run_param = RUN_PARAM["by_dir"]
    # 执行用例
    run_by(run_param)



# src/api/routes/authors.py
from flask import Blueprint
from flask import request
from ..utils.responses import response_with
from ..utils import responses as resp
from ..cases_execute import run_case
from ..utils.error import MissingError

test_routes = Blueprint("test_routes", __name__)


@test_routes.route('/run', methods=['POST'])
def test_cases():
    try:
        data = request.get_json()
        result = run_case(data)
        return response_with(resp.SUCCESS_201, value={"result":
                                                          result})
    except MissingError as e:
        print(e)
        return response_with(resp.INVALID_INPUT_422, message=e.__str__())
    except FileNotFoundError as e:
        print(e)
        return response_with(resp.INVALID_INPUT_423, message=e.__str__())



import re
import os
import sys

from requests import Session

template = """
args:
  - {method}
  - {api}
kwargs:
  -
    caseName: {caseName}
    {data_or_params}:
        {data}
validator:
  -
    json:
      successed: True
"""


def auto_gen_cases(swagger_url, project_name):
    """
    根据swagger返回的json数据自动生成yml测试用例模板
    :param swagger_url:
    :param project_name:
    :return:
    """
    res = Session().request('get', swagger_url).json()
    data = res.get('paths')

    workspace = os.getcwd()

    project_ = os.path.join(workspace, project_name)

    if not os.path.exists(project_):
        os.mkdir(project_)

    for k, v in data.items():
        pa_res = re.split(r'[/]+', k)
        dir, *file = pa_res[1:]

        if file:
            file = ''.join([x.title() for x in file])
        else:
            file = dir

        file += '.yml'

        dirs = os.path.join(project_, dir)

        if not os.path.exists(dirs):
            os.mkdir(dirs)

        os.chdir(dirs)

        for _k, _v in v.items():
            method = _k
            api = k
            caseName = _v.get('description')
            data_or_params = 'params' if method == 'get' else 'data'
            parameters = _v.get('parameters')

            data_s = ''
            try:
                for each in parameters:
                    data_s += each.get('name')
                    data_s += ': \n'
                    data_s += ' ' * 8
            except TypeError:
                data_s += '{}'

        file_ = os.path.join(dirs, file)

        with open(file_, 'w', encoding='utf-8') as fw:
            fw.write(template.format(
                method=method,
                api=api,
                caseName=caseName,
                data_or_params=data_or_params,
                data=data_s
            ))

        os.chdir(project_)


if __name__ == "__main__":
    auto_gen_cases("https://dl-admin.test.shopee.co.id/api/v1/swagger/doc.json", "discover")



import argparse
import os
import json
from typing import List
from ..utils.file_reader import JsonReader


def platform(key, value):
    if key == "list":
        from pyland import list_case
        list_case(value)
    elif key == "import":
        from pyland import update_case
        update_case(value)
    elif key == "run":
        from pyland import run_case
        run_case(value)
    else:
        raise ValueError("only chose from {list, import, run}")


def is_json(val):
    try:
        json.loads(val)
    except ValueError:
        return False
    return True


def main(args: List[str]) -> bool:
    parser = argparse.ArgumentParser(prog='pyland platform')
    platOpt = parser.add_mutually_exclusive_group(required=True)
    platOpt.add_argument('--import', help='import cases to platform')
    platOpt.add_argument('--list', help='list cases to platform')
    platOpt.add_argument('--run', help='run cases with platform')

    parsed_args = parser.parse_args(args)
    key, value = [(k, v) for k, v in vars(parsed_args).items() if v][0]

    # capture error input
    try:
        if is_json(value):
            value = json.loads(value)
        else:
            if os.path.isfile(value):
                value = JsonReader(value).data
            else:
                raise FileNotFoundError
    except FileNotFoundError as e:
        return f"FileNotFoundError: {value} not found"
    except Exception as e:
        return f"JSONDecodeError: {e}"

    # call platform command with json input
    return platform(key=key, value=value)



# -*- coding: utf-8 -*-
"""
文件读取。YamlReader读取yaml文件，ExcelReader读取excel。
"""
from yaml import safe_load_all as y_safe_local_all
from json import load as j_load
from os import path
from xlrd import open_workbook


class YamlReader(object):
    def __init__(self, file_path):
        if path.exists(file_path):
            self.file_path = file_path
        else:
            raise FileNotFoundError('文件{}不存在！'.format(file_path))
        self._data = None

    @property
    def data(self):
        # 如果第一次调用data，读取yaml文档，否则直接返回之前保存的数据
        if not self._data:
            with open(self.file_path, 'rb') as f:
                self._data = list(y_safe_local_all(f))  # load后是个generator，用list组织成列表
        return self._data


class JsonReader(YamlReader):
    @property
    def data(self):
        if not self._data:
            with open(self.file_path, 'rb') as f:
                self._data = j_load(f)
        return self._data


class RawReader(YamlReader):
    @property
    def data(self):
        if not self._data:
            with open(self.file_path, 'r', encoding="utf8") as f:
                self._data = f.read()
        return self._data


class SheetTypeError(Exception):
    pass


class ExcelReader():
    """
    读取excel文件中的内容。返回list。

    如：
    excel中内容为：
    | A  | B  | C  |
    | A1 | B1 | C1 |
    | A2 | B2 | C2 |

    如果 print(ExcelReader(excel, title_line=True).data)，输出结果：
    [{A: A1, B: B1, C:C1}, {A:A2, B:B2, C:C2}]

    如果 print(ExcelReader(excel, title_line=False).data)，输出结果：
    [[A,B,C], [A1,B1,C1], [A2,B2,C2]]

    可以指定sheet，通过index或者name：
    ExcelReader(excel, sheet=2)
    ExcelReader(excel, sheet='BaiDuTest')
    """
    def __init__(self, excel, sheet=0, title_line=True):
        if path.exists(excel):
            self.excel = excel
        else:
            raise FileNotFoundError('文件不存在！ ')
        self.sheet = sheet
        self.title_line = title_line
        self._data = list()

    @property
    def data(self):
        if not self._data:
            workbook = open_workbook(self.excel)
            if type(self.sheet) not in [int, str]:
                raise SheetTypeError('Please pass in <type int> or <type str>, not {0}'.format(type(self.sheet)))
            elif type(self.sheet) == int:
                s = workbook.sheet_by_index(self.sheet)
            else:
                s = workbook.sheet_by_name(self.sheet)

            if self.title_line:
                title = s.row_values(0)  # 首行为title
                for col in range(1, s.nrows):
                    # 依次遍历其余行，与首行组成dict，拼到self._data中
                    self._data.append(dict(zip(title, s.row_values(col))))
            else:
                for col in range(0, s.nrows):
                    # 遍历所有行，拼到self._data中
                    self._data.append(s.row_values(col))
        return self._data


if __name__ == '__main__':
    yml = '../config/config.yml'
    reader = YamlReader(yml)
    print(reader.data[1])

    xls = '../data/vid_feature_int384_sensenets_V6_20181211.xlsx'
    reader = ExcelReader(xls, title_line=False)
    print(reader.data[0][1])










"""
读取配置。采用yaml格式配置文件，也可以采用xml、ini等，需要在file_rader添加响应Reader处理。
❗️ 注意：基础脚本【config， support】，不能import更高级的脚本如logger， Sql
"""
import sys
import os
from .utils.file_reader import (
    YamlReader,
    JsonReader,
    RawReader
)
from .utils.file_writer import (
    YamlWriter,
    JsonWriter,
    RawWriter
)
from .utils.support import (
    to_list,
    eval_param,
    get_suffix_list
)

__all__ = [
    "BASE_PATH",
    "Config",
    "YamlParam",
    "com_params"
]

# current workspace path
# BASE_PATH = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
BASE_PATH = os.getcwd()


def is_base(pathname):
    # condition: BASE_PATH must have a sub-dir named with `data` and `test_suits`
    _data_path = os.path.join(pathname, 'data')
    _tests_path = os.path.join(pathname, 'test_suits')
    if os.path.isdir(_data_path) and os.path.isdir(_tests_path):
        return True
    else:
        return False


class Config:
    """
    get/put data from/to yaml/json/text
    Config(): default use ${workspace}/config/config.yml
    Config('api_config.yml'): use ${workspace}/config/api_config.yml
    Config('testAPI/interface/mod_2_video/test.yml'): use ${workspace}/testAPI/interface/mod_2_video/test.yml
    Config('/usr/data/images'): use /usr/data/images
    """

    def __init__(self, config='config.yml', index=0, path=None):
        """
        :param config: the config file path,
            - if absolute path like `/xx/config.yml`, will directly use it
            - if relative path like `xx/config.yml`, will Relative to the current workspace
            - if file name like `config.yml`, will Relative to the `config` foler in current worspace
            - if empty, will Relative to the `config/config.yml` foler in current worspace
        :param path: change workspace path
            - accept absolute/relative path
            - affect the variable DATA_PATH/LOG_PATH/CONFIG_PATH/REPORT_PATH and so on.
            - if config is abspath, will still use the config abspath
        :param index: yaml index
        """
        # use current workspace path as default BASE_PATH
        self.BASE_PATH = os.getcwd()

        # when path set, use path as BASE_PATH
        if path:
            if os.path.isdir(path):
                self.BASE_PATH = os.path.abspath(path)
            elif os.path.isfile(path):
                self.BASE_PATH = os.path.abspath(os.path.dirname(path))

        if is_base(self.BASE_PATH):
            # print(f"has found BASE_PATH: {self.BASE_PATH}")
            pass
        else:
            # print("try using config file to locate base path")
            _cfg_path = os.path.dirname(os.path.abspath(config))
            _base_path = os.path.dirname(_cfg_path)
            if os.path.isdir(os.path.join(_cfg_path, 'data')):
                self.BASE_PATH = _cfg_path
            if os.path.isdir(os.path.join(_base_path, 'data')):
                self.BASE_PATH = _base_path
        #
        # if path and os.path.isdir(path):
        #     self.BASE_PATH = os.path.abspath(path)
        #     sys.path.extend([self.BASE_PATH])
        #     # os.chdir(self.BASE_PATH)
        #     print(f"path: {path}\n{sys.path}\n{os.getcwd()}")
        # else:
        #     self.BASE_PATH = os.getcwd()

        self.CONFIG_PATH = os.path.join(self.BASE_PATH, 'config')
        self.DATA_PATH = os.path.join(self.BASE_PATH, 'data')
        self.LOG_PATH = os.path.join(self.BASE_PATH, 'log')
        self.REPORT_PATH = os.path.join(self.BASE_PATH, 'report')
        self.CONFIG_FILE = os.path.join(self.CONFIG_PATH, 'config.yml')
        self.DRIVERS_PATH = os.path.join(self.BASE_PATH, 'drivers')
        self.RESULT_PATH = os.path.join(self.BASE_PATH, "allure-result")

        self.COMMON_PATH = os.path.join(self.BASE_PATH, "common")
        sys.path.insert(0, self.COMMON_PATH)
        sys.path.insert(0, self.BASE_PATH)

        self.abspath = ''

        if config.startswith("/"):
            self.abspath = config
        elif '/' in config:
            self.abspath = os.path.join(self.BASE_PATH, config)
        else:
            self.abspath = os.path.join(self.CONFIG_PATH, config)

        try:
            self.config = self.get(index=index)
        except Exception as e:
            self.config = None

    def get(self, element=None, index=0):
        """
        get config element
        :param element: yaml element
        :param index: yaml is separated to lists by '---'
        :return:
        """
        try:
            self.config = YamlReader(self.abspath).data[index]
        except Exception as e:
            raise Exception("Yaml read error")

        if element:
            res = self.config.get(element)
        else:
            res = self.config
        return res

    def global_var(self, index=0):
        """
        Be careful! Transfer yaml key to global var name
        :return:
        """
        if index:
            self.config = YamlReader(self.abspath).data[index]

        for key in sorted(self.config.keys()):
            globals()[key] = self.config[key]

    def json_get(self):
        """get data from json file"""
        json_data = JsonReader(self.abspath).data
        return json_data

    def json_put(self, json_obj, abspath=None):
        """put json object to json file"""
        abspath = self.abspath if not abspath else abspath
        self.touch(abspath)
        JsonWriter(json_obj, self.abspath)

    def json_update(self):
        pass

    def raw_get(self):
        raw_data = RawReader(self.abspath).data
        return raw_data

    def raw_put(self, raw_obj):
        RawWriter(raw_obj, self.abspath)

    @classmethod
    def touch(cls, abspath):
        """When writing a file, to ensure that the path and files exist."""
        dirname = os.path.dirname(abspath)
        os.makedirs(dirname, exist_ok=True)
        with open(abspath, 'a'):
            os.utime(abspath, None)

    @classmethod
    def put(cls, yaml_obj, abspath):
        """Write the YAML format object to a new YAML file (overwritten)"""
        Config.touch(abspath)
        YamlWriter(yaml_obj, abspath, overwrite=True)

    @classmethod
    def update(cls, yaml_obj, abspath=None):
        """Write the YAML format object to an old YAML file (append)"""
        try:
            dest = YamlReader(abspath).data[0]
            if isinstance(dest, list) and isinstance(yaml_obj, list):
                YamlWriter(yaml_obj, abspath, overwrite=False)
            elif isinstance(dest, dict) and isinstance(yaml_obj, dict):
                dest.update(yaml_obj)
                YamlWriter(dest, abspath, overwrite=True)
            else:
                raise ValueError("Try to write different types of data to YAML files, please check! ")
        except FileNotFoundError:
            cls.put(yaml_obj, abspath)


VALID_TYPE = 'valid'
INVALID_TYPE = 'invalid'


class YamlParam(Config):
    """
    Read and Format Input Param from config
    """

    @staticmethod
    def parse_by_case_type(original_data, type=VALID_TYPE):
        # transfer `{a: {valid: []}, b: {valid: []}}` to `{a: [], b: []}`
        original_data = eval_param(original_data)
        param = {}
        param_list = []
        missing_type = "Pyland_Missing_" + type

        if isinstance(original_data, dict):
            type_value = original_data.get(type)
            if type_value:
                type_value = to_list(type_value)
                param_list = YamlParam.parse_by_case_type(type_value, type)
                return param_list
            else:
                # condition of missing the type key
                if VALID_TYPE in original_data or INVALID_TYPE in original_data:
                    return missing_type
                for key in original_data.keys():
                    _value = YamlParam.parse_by_case_type(original_data.get(key), type)
                    if _value != missing_type:
                        param[key] = _value
                return param

        elif isinstance(original_data, list):
            for value in original_data:
                param = YamlParam.parse_by_case_type(value, type)
                param_list.append(param)
            return param_list
        else:
            return original_data

    @staticmethod
    def dict_value_to_list(src_dict: dict):
        """
        To transfer a dict to a permutation lists
        :param src_dict: {key1: [a1,a2], key2: [b1,b2,b3], key3:[c1]}
        :return: [[a1,b1,c1], [a2,b2,c1], [a2,b3,c1]]
        """
        res = []
        over_key = set()
        while len(over_key) < len(src_dict):
            lr = len(res)
            res_i = {}
            for key in src_dict.keys():
                # ensure the value list is this format: [value]
                if src_dict[key] and isinstance(src_dict[key], list):
                    value_list = src_dict[key]
                elif src_dict[key] and isinstance(src_dict[key], dict):
                    value_list = YamlParam.dict_value_to_list(src_dict[key])
                else:
                    value_list = [src_dict[key]]

                # combine valid params
                if lr < len(value_list) - 1:
                    res_i[key] = value_list[lr]
                else:
                    over_key.add(key)
                    res_i[key] = value_list[-1]

                # disable key if value is 'x'
                if res_i[key] == 'x':
                    res_i.pop(key)
            if res_i not in res:
                res.append(res_i)
        return res

    @staticmethod
    def replace_json_with_dict(src_dict: dict, json: dict) -> list:
        """
        To combine yml wrong params with the template
        :param src: {arg1: [1,2,3]}
        :param json: {arg1: 0, arg2:9}
        :return: [{arg1: 1, arg2:9},{arg1: 2, arg2:9},{arg1: 3, arg2:9}]
        """
        res = []
        for key in src_dict.keys():
            # ensure the value list is this format: [value]
            if src_dict[key] and isinstance(src_dict[key], list):
                value_list = src_dict[key]
            elif src_dict[key] and isinstance(src_dict[key], dict):
                value_list = YamlParam.replace_json_with_dict(src_dict[key], json[key])
            else:
                value_list = [src_dict[key]]

            for val in value_list:
                tmp = json.copy()
                tmp[key] = val

                # disable key if value is 'x'
                if val == 'x':
                    tmp.pop(key)

                if tmp not in res:
                    res.append(tmp)
        return res

    def valid_params(self, element):
        """
        Get a combined valid data field list
        """
        # ele_valid = self.get(element).get('valid')
        ele = self.get(element)
        ele_valid = self.parse_by_case_type(ele, type='valid')
        if ele_valid and isinstance(ele_valid, dict):
            ele_valid = self.dict_value_to_list(ele_valid)
        self.ele_valid = ele_valid
        return self.ele_valid

    def invalid_params(self, element, template_file=None):
        """
        Get a combined invalid data field list
        """
        # ele_invalid = self.get(element).get('invalid')
        ele = self.get(element)
        ele_invalid = self.parse_by_case_type(ele, type='invalid')
        if ele_invalid and isinstance(ele_invalid, dict):
            try:
                if template_file:
                    template = Config(template_file).json_get()
                elif self.valid_params(element) and isinstance(self.ele_valid[-1], dict):
                    template = self.ele_valid[-1]
                ele_invalid = self.replace_json_with_dict(ele_invalid, template)
            except Exception as e:
                raise e
        self.ele_invalid = ele_invalid
        return self.ele_invalid

    def combined_params(self):
        """
        Combine all PARAMs under this YML file, generate _combined_xx.yml file
        :return: its dictionary format data
        """
        res = {}
        try:
            all_params = self.config
            if not all_params:
                raise Exception("yaml read failed. check the yaml format")
            for key in all_params.keys():
                if key.startswith("param"):
                    valid = self.valid_params(key)
                    invalid = self.invalid_params(key)
                    res.update({f"{key}_valid": valid, f"{key}_invalid": invalid})
                    if valid == invalid:
                        res.update({f"{key}": all_params[key]})
                else:
                    res.update({f"{key}": all_params[key]})
            original_path = os.path.split(self.abspath)
            out_path = os.path.join(original_path[0], "_combined_" + original_path[1])
            self.put(res, out_path)
        except Exception as e:
            raise e
        return res


def com_params(yaml_files_list=None, path=None, strict=True):
    """
    :param path: find all yml files in this path
    :param yaml_files_list: read data from a list of yaml file abspaths
    :param strict: when files contain duplicated keys,
        if True, will raise error;
        if False, will use the last value of last file
    :return: an input parameter dictionary according to the list of YAML files
    """
    if path:
        if not os.path.isdir(path):
            return None
        if yaml_files_list:
            if isinstance(yaml_files_list, str):
                yaml_files_list = [yaml_files_list]
            if isinstance(yaml_files_list, list):
                yaml_files_list = [os.path.join(path, i) for i in yaml_files_list if isinstance(i, str)]
        else:
            yaml_files_list = get_suffix_list(path, suffix=".yml")
    else:
        if not yaml_files_list or not isinstance(yaml_files_list, list):
            return None

    param_dict = {}
    for file in yaml_files_list:
        yp = YamlParam(file)
        combined = yp.combined_params()
        if strict:
            for key in combined.keys():
                if param_dict.get(key):
                    raise ValueError(f"Duplicated Key {key} found, please check")
        param_dict.update(combined)
    return param_dict


def com_params_obj(yaml_files_list=None, path=None, strict=True, obj=None):
    """
    :param path: find all yml files in this path
    :param yaml_files_list: read data from a list of yaml file abspaths
    :param strict: when files contain duplicated keys,
        if True, will raise error;
        if False, will use the last value of last file
    :return: an object which has attributes same with yaml keys `param_***`
    """
    param_dict = com_params(yaml_files_list=yaml_files_list, path=path, strict=strict)
    if obj:
        params_obj = obj
    else:
        params_obj = Params() 
    objDictTool.to_obj(params_obj, **param_dict)
    return params_obj


class objDictTool:
    """to change a dict to an object, or change an object to a dict"""

    @staticmethod
    def to_obj(obj: object, **data):
        obj.__dict__.update(data)

    @staticmethod
    def to_dic(obj):
        dic = {}
        for fieldkey in dir(obj):
            fieldvaule = getattr(obj, fieldkey)
            if not fieldkey.startswith("__") and not callable(fieldvaule) and not fieldkey.startswith("_"):
                dic[fieldkey] = fieldvaule
        return dic


class Params():
    """ store all the params as an object"""
    pass

# def auto_combine_params(path=BASE_PATH):
#     yaml_files_list = get_suffix_list(path, suffix=".yml")
#     return com_params(yaml_files_list)


# # combine params automatically
# BASE_PATH = os.path.dirname(os.path.abspath(__file__))
# PARAM_PATH = os.path.join(BASE_PATH, "data/input_loads")
# COMBINED_PARAMS = auto_combine_params(PARAM_PATH)



# coding=utf8
from yaml import dump as y_dump
from json import dump as j_dump


class YamlWriter(object):
    def __init__(self, yaml_obj, yaml_file, overwrite=False):
        if isinstance(yaml_obj, dict) or isinstance(yaml_obj, list):
            if overwrite:
                with open(yaml_file, "w") as f:
                    y_dump(yaml_obj, f, default_flow_style=False)
            else:
                with open(yaml_file, "a") as f:
                    y_dump(yaml_obj, f, default_flow_style=False)
        else:
            raise ValueError("只接受字典写入YAML文件,或列表追加YAML文件")


class JsonWriter(object):
    def __init__(self, json_obj, json_file):
        if isinstance(json_obj, dict):
            with open(json_file, "w") as f:
                j_dump(json_obj, f)
        else:
            raise ValueError("只接受字典写入JSON文件")


class RawWriter(object):
    def __init__(self, raw_obj, raw_file):
        with open(raw_file, "w") as f:
            f.write(raw_obj)


if __name__ == "__main__":
    import yaml
    with open("../api/config/config.yml", "r") as yaml_file:
        yaml_obj = yaml.load(yaml_file.read())
        print(yaml_obj)
    yaml_obj["sql_data1"]["sql_port"] = 1234
    with open("../api/config/config.yml", "w") as f:
        y_dump(yaml_obj, f, default_flow_style=False)


# run.py
from .main import app as application


def run_server(host="0.0.0.0", port=5000, debug=True, use_reloader=True):
    application.run(port=port, host=host, use_reloader=use_reloader, debug=debug)


if __name__ == "__main__":
    application.run(port=5000, host="0.0.0.0", use_reloader=True, debug=True)



"""
添加用于接口测试的client，对于HTTP接口添加HTTPClient，发送http请求。
还可以封装TCPClient，用来进行tcp链接，测试socket接口等等。
"""
from requests import session
import allure
from ..log import logger
from json import dumps as json_dumps

# import socket
# import json


METHODS = ['GET', 'POST', 'PUT', 'DELETE', 'HEAD', 'TRACE', 'OPTIONS', 'CONNECT']


class UnSupportMethodException(Exception):
    """传入method参数不支持时，抛出异常"""
    pass


class HTTPClient(object):
    """
    http请求的client。初始化时传入url、method等，可以添加headers和cookies，但没有auth、proxy。

    >>> HTTPClient('http://www.baidu.com').send()
    <Response [200]>
    """

    def __init__(self, url, method='POST', headers=None, cookies=None, timeout=20):
        """headers/cookies均为字典类型。如headers={'Content_Type':'text/html'}"""
        if headers is None:
            headers = {
                "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:83.0) Gecko/20100101 Firefox/83.0"
            }
        self.url = url
        self.method = method.upper()
        self.headers = headers
        self.session = session()
        self.cookies = cookies
        self.timeout = timeout
        if self.method not in METHODS:
            raise UnSupportMethodException('不支持的method：{0}，请检查传入参数！ '.format(self.method))
        self.set_headers(headers)
        self.set_cookies(cookies)
        self.set_logger()
        # logger.debug("Header: {}".format(self.session.headers))
        # logger.debug("Cookies:{}".format(self.session.cookies))

    def set_headers(self, headers):
        if headers:
            self.session.headers.update(headers)

    def set_cookies(self, cookies):
        if cookies:
            self.session.cookies.update(cookies)

    def set_logger(self):
        if not hasattr(self, 'logger'):
            self.logger = logger

    def send(self, params=None, data=None, json=None, files=None, **kwargs):
        """
        调用request方法的常见参数
        :param params: 紧跟在url后面，组合为<请求地址>, 格式为键值对（字典）
        :param data: <请求体>，字典、字节、文件格式参数
        :param json: <请求体>，json格式参数
        :param files: <请求体>，上传文件
        :param kwargs: 比较复杂的参数，如代理、认证等，一般不需要手动赋值
        :return: 返回<响应结果>
        """
        response = self.session.request(method=self.method, url=self.url,
                                        params=params, data=data, json=json,
                                        files=files, timeout=self.timeout, **kwargs)
        # 日志呈现
        self.logger.debug(self.session.cookies)
        self.logger.info('请求开始 {0} {1}'.format(self.method, response.url))
        self.logger.debug('请求头: {}'.format(self.session.headers))
        for body in (params, data, json):
            if body and isinstance(body, dict):
                self.logger.debug('请求内容: {}'.format(body))
        self.logger.info('响应结果: {}'.format(response))
        self.logger.debug('响应文本: {}'.format(u'{}'.format(response.text)))
        self.logger.debug('响应头:{}'.format(response.headers))

        # 报告呈现
        with allure.step("接口请求"):
            allure.attach(f"{self.method}\t{self.url}", "请求地址")
            allure.attach(str(self.session.headers), "请求头")
            for body in (params, data, json, files):
                allure.attach(str(body), "请求参数") if body else ''
            allure.attach(str(response.status_code), "响应码")
            allure.attach(str(response.text), "响应内容")
            allure.attach(str(response.headers), "响应头")
        return response

    def close(self):
        """关闭连接"""
        self.session.close()
        self.logger.info('请求结束\n')


#
# class TCPClient(object):
#     """用于测试TCP协议的socket请求，对于WebSocket，socket.io需要另外的封装"""
#     def __init__(self, domain, port, timeout=30, max_receive=102400):
#         self.domain = domain
#         self.port = port
#         self.connected = 0  # 连接后置为1
#         self.max_receive = max_receive
#         self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#         self._sock.settimeout(timeout)
#
#     def connect(self):
#         """连接指定IP、端口"""
#         if not self.connected:
#             try:
#                 self._sock.connect((self.domain, self.port))
#             except socket.error as e:
#                 logger.error(e)
#             else:
#                 self.connected = 1
#                 logger.debug('TCPClient connect to {0}:{1} success.'.format(self.domain, self.port))
#
#     def send(self, data, dtype='str', suffix=''):
#         """向服务器端发送send_string，并返回信息，若报错，则返回None"""
#         if dtype == 'json':
#             send_string = json_dumps(data) + suffix
#         else:
#             send_string = data + suffix
#         self.connect()
#         if self.connected:
#             try:
#                 self._sock.send(send_string.encode())
#                 logger.debug('TCPClient Send {0}'.format(send_string))
#             except socket.error as e:
#                 logger.exception(e)
#
#             try:
#                 rec = self._sock.recv(self.max_receive).decode()
#                 if suffix:
#                     rec = rec[:-len(suffix)]
#                 logger.debug('TCPClient received {0}'.format(rec))
#                 return rec
#             except socket.error as e:
#                 logger.exception(e)
#
#     def close(self):
#         """关闭连接"""
#         if self.connected:
#             self._sock.close()
#             logger.debug('TCPClient closed.')


if __name__ == '__main__':
    HTTPClient('http://www.baidu.com/').send()



# # coding=utf8
#
# import os
# import unittest
#
# from utils import (log, extractor, config, file_reader, client, assertion,
#                    support, generator, funny)
#
#
# class TestLeader(object):
#
#     def __init__(self, **kwargs):
#         """initialize TestLeader.
#
#         :param kwargs:
#         """
#         self.unittest_runner = unittest.TextTestRunner(**kwargs)
#         self.test_loader = unittest.TestLoader()
#         self.summary = None
#
#
#
#
#
# # if __name__ == '__main__':
# #     REPORT_NAME = '{}-report.html'.format(time.strftime('%Y-%m-%d-%H-%M-%S'))
# #     report = os.path.join(REPORT_PATH, REPORT_NAME)
# #
# #     suite = unittest.TestSuite()
# #     tests = [TestLogin('test_login_success'), TestLogin('test_login_fail')]
# #     suite.addTests(tests)
# #     with open(report, 'wb') as f:
# #         runner = HTMLTestRunner(f, verbosity=2, title='测试框架', description='接口html报告')
# #         runner.run(suite)
# #
# #     _email = Config().get('email')
# #
# #     message1 = '这是今天的测试报告'
# #     message2 = open(report, 'r', encoding='utf-8').read()
# #
# #     e = Email(title=_email.get('title'),
# #               receiver=_email.get('receiver'),
# #               server=_email.get('server'),
# #               sender=_email.get('sender'),
# #               password=_email.get('password'),
# #               path=report,
# #               message='{0}\n{1}'.format(message1, message2)
# #               )
# #     e.send()
# #



# coding=utf8
from re import split
from argparse import ArgumentParser
from ..log import logger
from .support import socket_open
from ..config import Config


def url_http(ip, port):
    return "http://{}:{}".format(ip, port)


def ip_port(http_url):
    ip, port = split("[/ :]", http_url.strip("http://"))
    return ip, port


def check_server_argv(ip="127.0.0.1", port=8080):
    # 检查指定的IP端口是否连接正常，否则重新输入，更新IP、端口
    port = int(port)
    verify_server = input("检测到以下服务器信息\tIP:{}\tPort:{}\t是否重设IP/端口(yes/no): ".format(ip, port))
    if verify_server == "yes" or verify_server == "y":
        while True:
            ip = str(input("请输入服务IP地址："))
            port = int(input("请输入服务端口："))
            if ip and port:
                if socket_open(ip, port):
                    break
            else:
                logger.error("输入不合法，请重新输入！")
    return ip, port


def get_base_url(url_name='', index=0):
    # 从config/config.yaml读取IP和端口，读取不到则从命令行读取
    default_ip, default_port = "127.0.0.1", 8080
    default_url = url_http(default_ip, default_port)

    if url_name in locals().keys():
        local_url = locals()[url_name]
        logger.info("LOCAL {}: {}".format(url_name, local_url))
        return local_url
    else:
        arg = ArgumentParser("Test Tool.")
        arg.add_argument("--ip", default=default_ip, type=str, help="the ip of server.")
        arg.add_argument("--port", default=default_port, type=int, help="the port of server.")
        args = arg.parse_args()
        argv_url = url_http(args.ip, int(args.port))

        if argv_url == default_url:
            try:
                config_url = Config().get(url_name, index=index)
                if config_url:
                    logger.info("CONFIG {}: {}".format(url_name, config_url))
                    args.ip, args.port = ip_port(config_url)
                    return config_url
                else:
                    logger.error("config.yml missing defination of `{}`".format(url_name))
                    return False
            except:
                logger.info("not found config")
                re_ip, re_port = check_server_argv(args.ip, args.port)
                input_url = url_http(re_ip, re_port)
                logger.info("INPUT {}: {}".format(url_name, input_url))
                return input_url
        else:
            logger.info("ARGV {}: {}".format(url_name, argv_url))
            return argv_url

# VIDAPP_URL = get_base_url('XX_URL', index=0)



import os.path
import re
import sys
from .log import logger


def dfs_showdir(path, depth=0):
    """ tree the path """
    if depth == 0:
        print(f"./{path}")

    for item in os.listdir(path):
        if '.git' not in item:
            print("| " * depth + "+--" + item)

            newitem = path + '/' + item
            if os.path.isdir(newitem):
                dfs_showdir(newitem, depth + 1)


def create_scaffold(project_name, simple=False, sub=False, topo=False):
    """ create scaffold with specified project name.
    """

    def show_tree(prj_name):
        try:
            print(f"\n$ tree {prj_name} -a")
            dfs_showdir(prj_name)
            print("")
        except Exception as e:
            logger.warning(e.__str__())

    def str2hump(text):
        arr = filter(None, text.lower().split('_'))
        res = ''
        for i in arr:
            res = res + i[0].upper() + i[1:]
        return res

    if not project_name or type(project_name) is not str:
        logger.error(f"Project name missing or not string type")
        return 1
    if not re.match("^[A-Za-z0-9_-]*$", project_name):
        logger.error(
            f"Project name format error: can only include [A-Z] or [a-z] or [0-9] or [_] "
        )
        return 1
    if os.path.isdir(project_name):
        logger.warning(
            f"Project folder {project_name} exists, please specify a new project name."
        )
        show_tree(project_name)
        return 1
    elif os.path.isfile(project_name):
        logger.warning(
            f"Project name {project_name} conflicts with existed file, please specify a new one."
        )
        return 1

    logger.info(f"Create new project: {project_name}")
    print(f"Project Root Dir: {os.path.join(os.getcwd(), project_name)}\n")

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    create_folder(project_name)
    create_folder(os.path.join(project_name, "common"))
    create_folder(os.path.join(project_name, "config"))
    create_folder(os.path.join(project_name, "data"))
    create_folder(os.path.join(project_name, "log"))
    create_folder(os.path.join(project_name, "report"))
    create_folder(os.path.join(project_name, "data/input_loads"))
    create_folder(os.path.join(project_name, "test_suits"))

    create_file(os.path.join(project_name, "common", "__init__.py"))
    create_file(os.path.join(project_name, "test_suits", "__init__.py"))
    create_file(os.path.join(project_name, "__init__.py"))

    demo_common_argv_content = """\"\"\"
Content of common/common_argv.py
Any function here can be directly called by yaml file in the data/input_loads
Example of yaml content:
url: ${url_http(127.0.0.1, 5000)}
\"\"\"
import re


def url_http(ip, port):
    return 'http://{}:{}'.format(ip, port)


def ip_port(http_url):
    ip, port = re.split('[/ :]', http_url.strip('http://'))
    return ip, port
"""

    demo_config_yml_content = f"""# Content of config/config.yml
# Required: API Base URL Config
test_{project_name}_url: 'https://{project_name}.com'

# Required: Log Config 
log:
  file_name: test.log
  backup: 5
  console_level: INFO
  file_level: INFO
  pattern: '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

# Optional: SQL Server/DB Config
test_{project_name}_server: &test_{project_name}_server
  'sql_server': 127.0.0.1
  'sql_port': 3306
  'sql_user': root
  'sql_password':  123456

test_{project_name}_db1:
  <<: *test_{project_name}_server
  'db_name': 'db1'
"""

    demo_param_yml_content = """# Content of data/input_loads/parameterize.yml
# basic value (NOT start with `param_`,  just called in this yaml file, but ignored by pyland parameterize) 
name: &name
  valid:
    - '#test'
  invalid:
    - false
    - ${url_http(127.0.0.1, 5000)}  # dynamic data （func located in common.common_argv）

offset_limit: &offset_limit
  offset:
    valid: [0, 1, 1000, 0]
    invalid: [~, -1, true]  # ~ means None
  limit:
    valid: [10, 1000, 20]
    invalid: [~, 'x', true]  # 'x' means missing key

# Combination Params
param_0:  # Combination: combine all params to 2 lists, called with `param_0_valid` or `param_0_invalid`
    name:
        valid:
            - '#test'
        invalid:
            - false
            - ${url_http(127.0.0.1, 5000)}
    offset:
        valid: [0, 1, 1000, 0]
        invalid: [~, -1, true]
    limit:
        valid: [10, 1000, 20]
        invalid: [~, 'x', true]

param_0_duplicate:  # same with param_0
  name: *name
  <<: *offset_limit

# No Combination Params (means user combine and write params manually)
param_1: 'ONE VALUE'  # One Directly Constant, called with `param_1`

param_2:  # One Directly List: called with `param_2`
  - name: 'test1'
    offset: 0
    limit: 1
  - name: 'test2'
    offset: -1
    limit: 0

param_3:  # Two Directly Lists: called with `param_3_valid` AND `param_3_invalid` 
  valid:
    - name: 'test1'
      offset: 0
      limit: 1
  invalid:
    - name: 'test2'
      offset: -1
      limit: 0
    - name: 'test3'
      offset: 0
      limit: -1
"""

    demo_const_content = f"""\"\"\"
Content of const.py
Global Constants Definitions.
\"\"\"
import os
from pyland import (
    Config,
    Logger
)

__all__ = [
    'Const',
    'cfg',
    'logger',
    'ParamSets'
]


class Const:
    \"\"\"Global Constants\"\"\"

    # relative to the dir `config`
    DEFAULT_CONF_PATH: str = 'config.yml'

    # relative to the dir `data`
    DEFAULT_PARAM_PATH: str = 'input_loads/parameterize.yml'
    DEFAULT_COOKIE_PATH: str = 'input_loads/cookies'

    # key name of DEFAULT_CONF_PATH
    DEFAULT_URL: str = 'test_{project_name}_url'
    DEFAULT_LOG: str = 'log'
    DEFAULT_MYSQL: str = ''

    # email or name
    AUTHOR: str = ''


# global base path is based on this file
_file_path = os.path.dirname(os.path.abspath(__file__))

# cfg.config: config content
# cfg.DATA_PATH: locate `data` path
cfg = Config(path=_file_path, config=Const.DEFAULT_CONF_PATH)

# logger.info()
logger = Logger(config=cfg.config[Const.DEFAULT_LOG], log_path=cfg.LOG_PATH)


# cookie content
# cookies = Config(path=cfg.DATA_PATH, config=Const.DEFAULT_COOKIE_PATH).raw_get()

# mysql config, usage as following:
# > from pyland import Sql
# > Sql(mysql_server).query()
# mysql_server = cfg.config[Const.DEFAULT_MYSQL]


class ParamSets:
    \"\"\"params defined here\"\"\"
    param_0_valid: list
    param_0_invalid: list

"""

    demo_services_topo_content = f"""\"\"\"
Content of services.py
API Services Definitions.
\"\"\"
import szqa_framework
import allure
from .const import *
from pyland import PRequest


class {str2hump(project_name)}(szqa_framework.TopoHttp, PRequest):
    \"\"\"
    {str2hump(project_name)} APIs.
    \"\"\"

    def __init__(self):
        super().__init__()                              # required
        self.base_url = cfg.config[Const.DEFAULT_URL]   # required
        self.config = cfg.config                        # optional
        self.base_path = cfg.BASE_PATH                  # optional
        self.data_path = cfg.DATA_PATH                  # optional

    @allure.step('api - 0')  # optional: request step name
    def get_{project_name}(self, json, status='PASS'):
        api_url = ''  
        method = 'GET'  
        res = self.send_request(api_url, method, params=json, status=status)  # GET method: params
        return res 
"""

    demo_services_content = f"""\"\"\"
Content of services.py
API Services Definitions.
\"\"\"
import os
import allure
from .const import *
from pyland import (
    PRequest,
    extract
)


class {str2hump(project_name)}(PRequest):
    \"\"\"
    {str2hump(project_name)} APIs.
    \"\"\"

    def __init__(self):
        super().__init__()  # required
        self.base_url = cfg.config[Const.DEFAULT_URL]  # required
        self.logger = logger
        self.config = cfg.config
        self.base_path = cfg.BASE_PATH
        self.data_path = cfg.DATA_PATH

    @allure.step('api - 0')  # request step name
    def get_{project_name}(self, json, status='PASS'):
        api_url = ''  
        method = 'GET'  
        res = self.send_request(api_url, method, params=json, status=status)  # GET method: params
        return res 

    @allure.step('api - 1')
    def post_{project_name}(self, json, status='PASS'):
        api_url = ''  
        method = 'POST'
        res = self.send_request(api_url, method, json=json, status=status)  # POST method: json
        self.id = extract('result.id', res)  # extract from API 1, use in API 2
        return res  

    @allure.step('api - 2')
    def put_{project_name}(self, json, status='PASS'):
        api_url = ''
        method = 'PUT'
        if json.get('path') and isinstance(json['path'], str):
            json['path'] = os.path.join(self.data_path, json['path'])
        with open(json['path'], 'rb') as f:
            res = self.send_request(api_url, method, json=json, files={{'name': f}}, status=status)  # File Type Request
        return res

    @allure.step('api - 3')
    def delete_{project_name}(self, json, status='PASS'):
        api_url = f'{{self.id}}'  # extract from API 1, use in API 2
        method = 'DELETE'
        res = self.send_request(api_url, method, status=status)  
        return res
"""

    demo_conftest_01_content = f"""\"\"\"
Content of test_suits/conftest.py
pytest conftest, setup or teardown hooks
\"\"\"
import pytest
from ..services import logger
from ..services import {str2hump(project_name)}


@pytest.fixture(scope='function')
def env_api():
    logger.info('准备测试，执行前置接口')
    {project_name} = {str2hump(project_name)}()
    yield {project_name}
    logger.info('结束测试，执行后置接口')

"""

    demo_test_topo_01_content = f"""\"\"\"
Content of test_suits/test_01.py 
\"\"\"
import os
import szqa_framework
from pyland import extract
from pyland import com_params_obj
from ..services import {str2hump(project_name)}
from ..const import *

__author__ = Const.AUTHOR

# Transfer params to Object attributes
MyParam = ParamSets()
com_params_obj(path=cfg.DATA_PATH, yaml_files_list=Const.DEFAULT_PARAM_PATH, obj=MyParam)


class Test{str2hump(project_name)}1({str2hump(project_name)}):
    INPUT = {{'param1': MyParam.param_0_valid}}  # 声明当前case接受一个key为param1的传惨

    @szqa_framework.case_setting(paramesize=['param1'])
    def test_case(self):
        self.logger.info('Step1: demo step')
        self.logger.debug(f'This is the input [{{self.input[\"param1\"]}}]') 
        res = self.get_demo_api(self.input['param1'])
        if res is not None:
            self.result.add_result(self.result.PASS, item='step1', comment='demo test')  


class Test{str2hump(project_name)}2({str2hump(project_name)}):
    @szqa_framework.case_setting(input={{'param2': MyParam.param_0_invalid}}, paramesize=['param2'])
    def test_case(self):
        self.logger.info('Step1: demo step')
        self.logger.debug(f'This is the input [{{self.input}}]')  # input属性是外部传入的参数
        res = self.get_demo_api(self.input['param2'])
        assert res is not None


if __name__ == '__main__':
    import os

    szqa_framework.set_env_name('api_env')
    root_path = cfg.BASE_PATH
    case_path = os.path.join(root_path, 'test_suits/test_01.py')
    case_name_1 = 'TopoHttp-Test{str2hump(project_name)}1.test_case'
    case_name_2 = 'TopoHttp-Test{str2hump(project_name)}2.test_case'

    print(szqa_framework.get_case_tree(root_path))
    print(szqa_framework.run_one_case(case_path, case_name_1, root_path))
    print(szqa_framework.run_one_case(case_path, case_name_2, root_path))
    # case_executor = szqa_framework.Executor(case_path, case_name, root_path)
    # case_executor.run_case(case_name)
    # print(case_executor.get_case_result())
    # print(case_executor.get_case_output())


"""

    demo_test_01_content = f"""\"\"\"
Content of test_suits/test_01.py 
\"\"\"
import allure
import pytest
from pyland import extract
from pyland import com_params_obj
from ..services import {str2hump(project_name)}
from ..const import *

__author__ = Const.AUTHOR

# Transfer params to Object attributes
MyParam = ParamSets()
com_params_obj(path=cfg.DATA_PATH, yaml_files_list=Const.DEFAULT_PARAM_PATH, obj=MyParam)


@allure.epic('epic {str2hump(project_name)}')  
@allure.feature('feature {str2hump(project_name)} ') 
class Test{str2hump(project_name)}(object):
    \"\"\"test cases\"\"\"

    @allure.title('Get {str2hump(project_name)} Valid')   
    @allure.severity('normal')  
    @pytest.mark.parametrize("INPUT", MyParam.param_0_valid)
    def test_get_{project_name}_valid(self, INPUT):
        {project_name} = {str2hump(project_name)}()
        res = {project_name}.get_{project_name}(INPUT)
        success = extract('success', res)
        assert success

    @allure.title('Get {str2hump(project_name)} Invalid')  
    @pytest.mark.parametrize("INPUT", MyParam.param_0_invalid)
    def test_get_{project_name}_invalid(self, INPUT):
        {project_name} = {str2hump(project_name)}()
        res = {project_name}.get_{project_name}(INPUT)
        success = extract('success', res)
        assert not success
"""

    demo_simple_test_01_content = f"""\"\"\"
Content of simple test_suits/test_01.py 
\"\"\"


def test_get_{project_name}_valid():
    \"\"\"test cases content\"\"\"
    assert True

"""

    demo_run_py_content = f"""\"\"\"Content of run.py\"\"\"
from pyland import run_by

# *按需更改*
RUN_PARAM = {{
    "collect": ['test_suits', '--collect-only'],
    "by_dir": ['test_suits'],
    "by_mod": ['test_suits/test_01.py'],
    "by_class": ['test_suits/test_01.py::Test{str2hump(project_name)}'],
    "by_func": ['test_suits/test_01.py::Test{str2hump(project_name)}::test_get_{project_name}_valid'],
    "by_search": ['-k', 'test_get_{project_name}_valid'],
    "by_severity": ['--allure-severities', 'blocker'],
    "by_feature": ['--allure-features', 'feature {str2hump(project_name)}'],
    "by_story": ['--allure-stories', 'story {str2hump(project_name)}'],
}}

if __name__ == "__main__":
    # *按需调用*
    run_param = RUN_PARAM["by_dir"]
    # 执行用例
    run_by(run_param)
"""

    demo_main_py_content = """\"\"\"
Content of main.py
Entry point for third party to call. Afford 2 calling choices
\"\"\"

# \"\"\"
# choice 1
# same as executing command `pyland` at term, example following:
# ❯ python main.py
# usage: pyland [-h] [--version] [--no-color] {init,server,platform}
# pyland: error: the following arguments are required: command, args
# \"\"\"
#
# if __name__ == "__main__":
#     import pyland
#     sys.exit(pyland.main())


\"\"\"
choice 2
same as executing command `pyland platform` at term
❯ python main.py
usage: Process some integrated operations with Auto platform.
       [-h] (--import IMPORT | --list LIST | --run RUN)
Process some integrated operations with Auto platform.: error: one of the arguments --import --list --run is required
\"\"\"

if __name__ == '__main__':
    import pyland
    import sys

    sys.exit(pyland.main(['platform', *sys.argv[1:]]))

"""

    demo_conftest_py_content = """\"\"\"
[Important!!!  DO NOT TOUCH!!!]
\"\"\"
import os
from pyland import Config


pytest_plugins = 'pytester'


def pytest_addoption(parser):
    parser.addoption('--case-manage', required=False, help='analysis cases to manage by platform')


def pytest_generate_tests(metafunc):
    res = {}
    res['extra'] = {}
    res['extra']['params'] = {}

    for param in metafunc.fixturenames:
        if param.startswith('param'):
            if not param.endswith('valid'):
                param = param + '_valid'
            if hasattr(metafunc.module, 'combined'):
                param_pool = metafunc.module.combined[param]
                metafunc.parametrize(param, param_pool)
                # res['extra']['params'][param] = param_pool
            else:
                raise ValueError(f"param {param} set, but not found `combined`")

    project_path = metafunc.config.getoption('--case-manage')

    if project_path:
        for mark in metafunc.definition.iter_markers():
            if mark.name == 'allure_display_name':
                res['name'] = mark.args[0]
            elif mark.name == 'allure_label':
                label_type = mark.kwargs['label_type']
                label_value = mark.args[0]
                res['extra'][label_type] = label_value

        res['extra']['nodeID'] = metafunc.definition.nodeid
        res['extra']['path'] = metafunc.definition.location[0]
        res['extra']['module'] = metafunc.module.__name__
        if hasattr(metafunc.module, '__author__'):
            res['author'] = metafunc.module.__author__
        else:
            res['author'] = 'has no author yet'
        res['extra']['class'] = metafunc.cls.__name__ if metafunc.cls else None
        res['extra']['function'] = metafunc.function.__name__
        res['description'] = 'has no description yet' if not metafunc.function.__doc__ else metafunc.function.__doc__
        res['primary_key'] = 'extra.nodeID'
        res['category'] = 'pyland'
        res['priority'] = res['extra'].get('severity') if res['extra'].get('severity') else ''

        cfg = Config(path=project_path)
        case_manage_yml = os.path.join(cfg.DATA_PATH, '.tmp_case_manage.yml')
        cfg.update([res], case_manage_yml)

"""

    demo_input_project_content = """{
    "project_id": 0,
    "project_path": "",
    "host": ""
}
"""

    demo_input_case_content = f"""{{
    "project_path": "",
    "cases": [
        {{
            "case_id": 1,
            "extra": {{
                "class": "Test{str2hump(project_name)}",
                "feature": "{project_name} feature 01",
                "function": "test_get_{project_name}_valid",
                "module": "test_suits.test_01",
                "nodeID": "test_suits/test_01.py::Test{str2hump(project_name)}::test_get_{project_name}_valid",
                "params": {{}},
                "path": "test_suits/test_01.py",
                "severity": "blocker",
                "story": "{project_name} story 01"
            }}
        }}
    ],
    "project_name": "{project_name}",
    "category": "pyland",
    "output_path": "output.json",
    "runtime_args": {{
        "-n": "auto"
    }}
}}
"""

    create_file(os.path.join(project_name, "common", "common_argv.py"), demo_common_argv_content)
    create_file(os.path.join(project_name, "config", "config.yml"), demo_config_yml_content)
    create_file(os.path.join(project_name, "data/input_loads", "parameterize.yml"), demo_param_yml_content)
    create_file(os.path.join(project_name, "const.py"), demo_const_content)

    if not sub and not topo:
        create_file(os.path.join(project_name, "main.py"), demo_main_py_content)
        create_file(os.path.join(project_name, "conftest.py"), demo_conftest_py_content)

    if simple:
        create_file(os.path.join(project_name, "test_suits", "test_01.py"), demo_simple_test_01_content)
    else:
        create_file(os.path.join(project_name, "test_suits", "test_01.py"), demo_test_01_content)

    if topo:
        create_file(os.path.join(project_name, "services.py"), demo_services_topo_content)
        create_file(os.path.join(project_name, "test_suits", "test_01.py"), demo_test_topo_01_content)
    else:
        create_file(os.path.join(project_name, "services.py"), demo_services_content)
        create_file(os.path.join(project_name, "test_suits", "conftest.py"), demo_conftest_01_content)
        create_file(os.path.join(project_name, "run.py"), demo_run_py_content)
        create_file(os.path.join(project_name, "data/input_loads", "input_project.json"), demo_input_project_content)
        create_file(os.path.join(project_name, "data/input_loads", "input_case.json"), demo_input_case_content)

    show_tree(project_name)
    return 0


def main_scaffold(project, simple, sub, topo):
    sys.exit(create_scaffold(project, simple, sub, topo))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import mysql.connector
from .log import logger
from .config import Config
from .utils.error import Sqlerror


class Sql(object):
    def __init__(self, sql_server="discover_services_id_test", db=None, charset="utf8"):
        if isinstance(sql_server, str):
            sql_server = Config().get(sql_server)

        self.sql_server = sql_server.get('sql_server')
        self.sql_port = sql_server.get('sql_port')
        self.sql_user = sql_server.get('sql_user')
        self.sql_password = sql_server.get('sql_password')
        self.db_name = sql_server.get('db_name') if not db else db
        self.charset = charset
        # self.connect()
        #
        # # 使用cursor()方法获取操作游标
        # self.cursor = self.db.cursor()
        # self.cursor.execute("SET NAMES utf8mb4")

    def connect(self, db_name=None):
        if db_name is None:
            db_name = self.db_name
        logger.debug('连接到mysql服务器{}'.format(self.sql_server))
        self.db = mysql.connector.connect(host=self.sql_server, port=self.sql_port,
                                          user=self.sql_user, passwd=self.sql_password,
                                          database=db_name, use_unicode=True, buffered=True, charset=self.charset)
        # 使用cursor()方法获取操作游标
        self.cursor = self.db.cursor()
        self.cursor.execute("SET NAMES utf8mb4")
        return self.db

    def exec(self, sql):
        try:
            self.connect()
            self.cursor.execute(sql)
            self.db.commit()
            logger.info('sql执行成功')
            self.close()
        except:
            err = 'sql执行失败，回滚数据库'
            logger.error(err)
            self.db.rollback()
            raise Sqlerror(err)

    def query(self, sql):
        try:
            self.connect()
            logger.debug(sql)
            self.cursor.execute(sql)

            results = []
            row = self.cursor.fetchone()
            while row is not None:
                # logger.debug(row)
                results.append(row)
                row = self.cursor.fetchone()

            # results = self.cursor.fetchall()
            logger.debug('查询结果为：{}'.format(results))
            self.close()
            return results
        except:
            err = "Error: unable to fecth data"
            logger.error(err)
            raise Sqlerror(err)

    def load_sql(self, abs_sql_file):
        """
        Load sql file to mysql server.
        Sql file must only contains sql cmd (not support comment), every cmd should end with ';' and a line break.
        注意：这里埋了一个坑，sql文件如果结尾处没有空行，执行不成功且不会报错
        """
        self.connect()
        logger.debug("load {} ".format(abs_sql_file))
        with open(abs_sql_file, 'r', encoding='utf-8') as file_sql:
            file_sql_video = file_sql.read().split(';\n')[:-1]
            # logger.debug(sql_video)
            sql_list = [x.replace('\n', '') if '\n' in x else x for x in file_sql_video]
            # logger.debug(sql_list)

        for sql_item in sql_list:
            self.exec(sql_item)

        self.close()

    def batch_single_query(self, query_state, batch_size=1):
        try:
            self.connect()
            self.cursor.execute(query_state)
            i = 0
            query_batch_list = []

            while True:
                row = self.cursor.fetchmany(batch_size)
                if not row:
                    break
                single_list = []
                for single_element in row:
                    single_list.append(single_element[0])
                query_batch_list.append(single_list)
            self.close()
            return query_batch_list
        except:
            err = "Error: unable to fecth data"
            logger.error(err)
            raise Sqlerror(err)

    def pre_data(self, data, strip=True):
        """
        strip data
        replace [' " \\] to [\' \" \\\\]
        """
        if strip:
            data = data.strip()

    def close(self):
        if self.cursor:
            self.cursor.close()
        if self.db:
            self.db.close()


if __name__ == '__main__':
    query_camera_count = ""
    mysql = Sql()
    count = mysql.query(query_camera_count)[0][0]
    logger.info(count)
    mysql.close()



from setuptools import setup


INSTALL_REQUIRES = [
    "pytest==6.0.1",
    "allure-pytest==2.8.18",
    "requests==2.23.0",
    "PyYAML>=5.3.1",
    "xlwt==1.3.0",
    "xlrd==1.2.0",
    "mysql-connector-python>=8.0.20",
    "jmespath==0.10.0",
    "selenium",
    "Faker",
    "Pillow",
    "pypng==0.0.20",
    "pytest-xdist==2.1.0",
    "Flask",
    "Jinja2",
    "flask_migrate",
    "flask_cors",
    "Flask-SQLAlchemy",
    "flask_migrate",
    "flask_restful",
    "fnv",
    "colorama",
    "importlib_metadata>=4.5.0"
]


def main():
    setup(
        use_scm_version={"write_to": "src/_pyland/_version.py"},
        setup_requires=["setuptools-scm", "setuptools>=40.0"],
        package_dir={"": "src"},
        # fmt: on
        install_requires=INSTALL_REQUIRES,
    )


if __name__ == "__main__":
    main()




# PYTHON_ARGCOMPLETE_OK
"""pyland: launch auto test framework with Pytest."""
from _pyland.api.run import run_server
from _pyland.thirdparty import CaseManage
from _pyland.thirdparty import GroupManage
from _pyland.thirdparty import ProjectManage
from _pyland.thirdparty import ResultManage
from _pyland.utils import injson
from _pyland.utils import support
from _pyland.utils import client
from _pyland.utils import assertion
from _pyland.utils import error
from _pyland.cases_collect import list_case
from _pyland.cases_collect import update_case
from _pyland.cases_execute import run_case
from _pyland.config import Config
from _pyland.config import YamlParam
from _pyland.config import com_params
from _pyland.config import com_params_obj
from _pyland.extractor import extract
from _pyland.log import Logger
from _pyland.log import logger
from _pyland.pre_request import PRequest
from _pyland.pytest_run import run_by
from _pyland.sql import Sql
from _pyland.cmd import console_main
from _pyland.cmd import main
from _pyland import exceptions
from _pyland import commands
from _pyland import config
from _pyland import utils
from _pyland.scaffold import main_scaffold


__all__ = [
    "injson",
    "client",
    "assertion",
    "extract",
    "Config",
    "logger",
    "Logger",
    "run_by",
    "list_case",
    "update_case",
    "run_case",
    "console_main",
    "main",
    "exceptions",
    "run_server",
    "commands",
    "utils",
    "config",
    "error",
    "com_params",
    "com_params_obj",
    "support"
]

import importlib_metadata

metadata = importlib_metadata.metadata("pyland")

__title__ = metadata["name"]
__summary__ = metadata["summary"]
__uri__ = metadata["home-page"]
__version__ = metadata["version"]
__author__ = metadata["author"]
__email__ = metadata["author-email"]
__license__ = metadata["license"]

if __name__ == "__main__":
    import pyland
    raise SystemExit(pyland.console_main())



# -*- coding: utf-8 -*-
"""
日志类。读取配置文件，定义日志级别、日志文件名、日志格式。
一般直接把logger import进去
from utils.log import logger
logger.info('test log')
"""
import logging
import os
from time import strftime
from .config import Config, BASE_PATH

__all__ = [
    "Logger",
    "logger"
]

# default log path and config sample
LOG_PATH = os.path.join(BASE_PATH, 'log')
LOG_CONFIG = {
    "backup": 5,
    "console_level": "INFO",
    "file_level": "INFO",
    "file_name": "test.log",
    "pattern": "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    "log_path": LOG_PATH
}


class Logger(object):
    def __init__(self, logger_name='PYTEST-REPORT', config=None, log_path=None):
        self.logger = logging.getLogger(logger_name)
        logging.root.setLevel(logging.NOTSET)

        ConfigLog = None
        try:
            cfg = Config()
            CONFIGS = cfg.get()
            ConfigLog = CONFIGS["log"]
            ConfigLog["log_path"] = cfg.LOG_PATH
        except:
            ConfigLog = None

        if config:
            self.config = config
        elif ConfigLog:
            self.config = ConfigLog
        else:
            self.config = LOG_CONFIG

        self.console_output_level = self.config.get('console_level')  # WARNING
        self.file_output_level = self.config.get('file_level')  # INFO
        self.backup_count = self.config.get('backup')  # 5
        self.formatter = logging.Formatter(self.config.get('pattern'))
        self.log_path = log_path if log_path else self.config.get('log_path')
        self.log_file_name = os.path.join(self.log_path,
                                          '{0}-{1}'.format(strftime('%Y-%m-%d'), self.config.get('file_name')))

        os.makedirs(self.log_path, exist_ok=True)

    def __logger(self, level, message):
        # 创建一个FileHandler，用于写到本地
        file_handler = logging.FileHandler(self.log_file_name, 'a', encoding='utf-8')
        file_handler.setFormatter(self.formatter)
        file_handler.setLevel(self.file_output_level)
        self.logger.addHandler(file_handler)

        # 创建一个StreamHandler,用于输出到控制台
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(self.formatter)
        console_handler.setLevel(self.console_output_level)
        self.logger.addHandler(console_handler)

        if level == 'info':
            self.logger.info(message)
        elif level == 'debug':
            self.logger.debug(message)
        elif level == 'warning':
            self.logger.warning(message)
        elif level == 'error':
            self.logger.error(message)
        # 这两行代码是为了避免日志输出重复问题
        self.logger.removeHandler(console_handler)
        self.logger.removeHandler(file_handler)
        # 关闭打开的文件
        file_handler.close()

    def debug(self, message):
        self.__logger('debug', message)

    def info(self, message):
        self.__logger('info', message)

    def warning(self, message):
        self.__logger('warning', message)

    def error(self, message):
        self.__logger('error', message)


logger = Logger()
logging.getLogger('faker').setLevel(logging.ERROR)



import argparse
from typing import List
from pyland import run_server


def main(args: List[str]) -> bool:
    parser = argparse.ArgumentParser(
        prog="pyland server",
        description="run local server to debug")
    parser.add_argument(
        "--port",
        default='5000',
        required=False,
        help="local server port",
    )

    parsed_args = parser.parse_args(args)

    # print(vars(parsed_args))
    return run_server(port=parsed_args.port)
